@@ -164,7 +164,7 @@ yq <- function(..., quiet = FALSE, tz = NULL, locale = Sys.getlocale("LC_TIME"))
 ##' ## ** internationalization **
 ##' \dontrun{
 ##' x_RO <- "Ma 2012 august 14 11:28:30 "
-##' ymd_hms(x_RO, locale = "ro_RO.utf8")
+##'   ymd_hms(x_RO, locale = "ro_RO.utf8")
 ##' }
 ##'
 ##' ## ** truncated time-dates **
@@ -462,15 +462,15 @@ hms <- function(..., quiet = FALSE, roll = FALSE) {
 ##' @param locale locale to be used, see \link{locales}. On linux systems you
 ##'   can use `system("locale -a")` to list all the installed locales.
 ##' @param select_formats A function to select actual formats for parsing from a
-##'   set of formats which matched a training subset of `x`. it receives a named
+##'   set of formats which matched a training subset of `x`. It receives a named
 ##'   integer vector and returns a character vector of selected formats. Names
 ##'   of the input vector are formats (not orders) that matched the training
 ##'   set. Numeric values are the number of dates (in the training set) that
 ##'   matched the corresponding format. You should use this argument if the
 ##'   default selection method fails to select the formats in the right
-##'   order. By default the formats with most formating tockens (\%) are
-##'   selected and \%Y counts as 2.5 tockens (so that it has a priority over
-##'   \%y\%m). Se examples.
+##'   order. By default the formats with most formating tokens (\%) are
+##'   selected and \%Y counts as 2.5 tokens (so that it has a priority over
+##'   \%y\%m). See examples.
 ##' @param exact logical. If `TRUE`, the `orders` parameter is interpreted as an
 ##'   exact `strptime()` format and no training or guessing are performed
 ##'   (i.e. `train`, `drop` parameters are irrelevant).
@@ -567,7 +567,7 @@ hms <- function(..., quiet = FALSE, roll = FALSE) {
 ##'
 ##' parse_date_time(c("27-09-13", "27-09-2013"), "dmy", select_formats = my_select)
 ##'
-##' ## ** invalid times with "fast" parcing **
+##' ## ** invalid times with "fast" parsing **
 ##' parse_date_time("2010-03-14 02:05:06",  "YmdHMS", tz = "America/New_York")
 ##' parse_date_time2("2010-03-14 02:05:06",  "YmdHMS", tz = "America/New_York")
 ##' parse_date_time2("2010-03-14 02:05:06",  "YmdHMS", tz = "America/New_York", lt = TRUE)
@@ -733,7 +733,7 @@ fast_strptime <- function(x, format, tz = "UTC", lt = TRUE, cutoff_2000 = 68L) {
     ## C PARSER:
     out <- fast_strptime(x, fmt, tz = "UTC", lt = FALSE)
 
-    if ( tz != "UTC" ){
+    if ( tz != "UTC" ) {
       out <-
         if( zpos > 0 ){
           if( !quiet )
@@ -869,4 +869,15 @@ fast_strptime <- function(x, format, tz = "UTC", lt = TRUE, cutoff_2000 = 68L) {
   }else as.character(x)
 }
 
+.parse_iso_dt <- function(x, tz) {
+  parse_date_time(x, orders = c("ymdTz", "ymdT", "ymd"), tz = tz, train = FALSE)
+}
+
+as_POSIXct <- function(x, tz) {
+  if (is.character(x))
+    .parse_iso_dt(x, tz = tz)
+  else if (!is.POSIXct(x))
+    as.POSIXct(x, tz = tz)
+  else x
+}
 ## parse.r ends here
